@@ -331,7 +331,7 @@ const int units_to_msgno[] = {
     /*ft*/428,
     -1, /* yards */
     /*°*/344,
-    /*ᵍ*/76,
+    /*ᵍ*/345,
     /*%*/96,
     -1 /* minutes */
 };
@@ -702,9 +702,9 @@ cmd_begin(void)
 extern void
 free_settings(settings *p) {
    /* don't free default ordering or ordering used by parent */
-   reading *order = p->ordering;
+   const reading *order = p->ordering;
    if (order != default_order && (!p->next || order != p->next->ordering))
-      osfree(order);
+      osfree((reading*)order);
 
    /* free Translate if not used by parent */
    if (!p->next || p->Translate != p->next->Translate)
@@ -1260,6 +1260,7 @@ cmd_data(void)
    reading *new_order, d;
    unsigned long mUsed = 0;
    char *style_name;
+   int old_style = pcs->style;
 
    /* after a bad *data command ignore survey data until the next
     * *data command to avoid an avalanche of errors */
@@ -1277,6 +1278,13 @@ cmd_data(void)
    }
 
    if (style == STYLE_UNKNOWN) {
+      if (!buffer[0]) {
+	 /* "*data" reinitialises the current style - for *data passage that
+	  * breaks the passage.
+	  */
+	 style = old_style;
+	 goto reinit_style;
+      }
       /* TRANSLATORS: e.g. trying to refer to an invalid FNORD data style */
       compile_diagnostic(DIAG_ERR|DIAG_BUF|DIAG_SKIP, /*Data style “%s” unknown*/65, buffer);
       return;
@@ -1507,13 +1515,14 @@ cmd_data(void)
    /* don't free default ordering or ordering used by parent */
    if (pcs->ordering != default_order &&
        !(pcs->next && pcs->next->ordering == pcs->ordering))
-      osfree(pcs->ordering);
+      osfree((reading*)pcs->ordering);
 
    pcs->style = style;
    pcs->ordering = new_order;
 
    osfree(style_name);
 
+reinit_style:
    if (style == STYLE_PASSAGE) {
       lrudlist * new_psg = osnew(lrudlist);
       new_psg->tube = NULL;
@@ -1653,7 +1662,7 @@ cmd_declination(void)
     if (v == HUGE_REAL) {
 	get_token_no_blanks();
 	if (strcmp(ucbuffer, "AUTO") != 0) {
-	    compile_diagnostic(DIAG_ERR|DIAG_SKIP|DIAG_COL, /*Expected number or 'AUTO'*/309);
+	    compile_diagnostic(DIAG_ERR|DIAG_SKIP|DIAG_COL, /*Expected number or “AUTO”*/309);
 	    return;
 	}
 	/* *declination auto X Y Z */
@@ -1661,7 +1670,7 @@ cmd_declination(void)
 	real y = read_numeric(fFalse);
 	real z = read_numeric(fFalse);
 	if (!pcs->proj) {
-	    compile_diagnostic(DIAG_ERR, /*Input coordinate system must be specified for '*DECLINATION AUTO'*/301);
+	    compile_diagnostic(DIAG_ERR, /*Input coordinate system must be specified for “*DECLINATION AUTO”*/301);
 	    return;
 	}
 	if (!proj_wgs84) {
