@@ -69,6 +69,9 @@
 // Order here needs to match order of export_format enum in export.h.
 
 const format_info export_format_info[] = {
+    { ".csv", /*CSV files*/101,
+      LABELS|ENTS|FIXES|EXPORTS|EXPORT_3D,
+      LABELS },
     { ".dxf", /*DXF files*/411,
       LABELS|LEGS|SURF|SPLAYS|STNS|PASG|XSECT|WALLS|MARKER_SIZE|TEXT_HEIGHT|GRID|FULL_COORDS,
       LABELS|LEGS|STNS },
@@ -88,7 +91,7 @@ const format_info export_format_info[] = {
       LEGS|SPLAYS|CENTRED|EXPORT_3D,
       LEGS },
     { ".kml", /*KML files*/444,
-      LABELS|LEGS|SPLAYS|PASG|XSECT|WALLS|ENTS|FIXES|EXPORTS|PROJ|EXPORT_3D,
+      LABELS|LEGS|SPLAYS|PASG|XSECT|WALLS|ENTS|FIXES|EXPORTS|PROJ|EXPORT_3D|CLAMP_TO_GROUND,
       LABELS|LEGS },
     /* TRANSLATORS: "Compass" and "Carto" are the names of software packages,
      * so should not be translated:
@@ -1337,6 +1340,11 @@ Export(const wxString &fnm_out, const wxString &title,
    bool need_bounds = true;
    ExportFilter * filt;
    switch (format) {
+       case FMT_CSV:
+	   filt = new POS(model.GetSeparator(), true);
+	   show_mask |= FULL_COORDS;
+	   need_bounds = false;
+	   break;
        case FMT_DXF:
 	   filt = new DXF(text_height);
 	   break;
@@ -1357,17 +1365,19 @@ Export(const wxString &fnm_out, const wxString &title,
        case FMT_JSON:
 	   filt = new JSON;
 	   break;
-       case FMT_KML:
-	   filt = new KML(model.GetCSProj().c_str());
+       case FMT_KML: {
+	   bool clamp_to_ground = (show_mask & CLAMP_TO_GROUND);
+	   filt = new KML(model.GetCSProj().c_str(), clamp_to_ground);
 	   show_mask |= FULL_COORDS;
 	   need_bounds = false;
 	   break;
+       }
        case FMT_PLT:
 	   filt = new PLT;
 	   show_mask |= FULL_COORDS;
 	   break;
        case FMT_POS:
-	   filt = new POS(model.GetSeparator());
+	   filt = new POS(model.GetSeparator(), false);
 	   show_mask |= FULL_COORDS;
 	   need_bounds = false;
 	   break;
